<?php

class PubSubHubBub {

    private $_hub_url = false;
    private $_params = array();

    public function __construct($hub_url) {
        $this->_hub_url = $hub_url;
    }

    private function param($k, $v) {
        $this->_params[$k] = $v;
    }

    private function _post() {
        $query_params = array();
        foreach ($this->_params as $k => $v) {
            $query_params[] = $k . '=' . urlencode($v);
        }
        $data = join('&', $query_params);
        $options = array(CURLOPT_URL => $this->_hub_url,
                         CURLOPT_POST => true,
                         CURLOPT_POSTFIELDS => $data,
                         CURLOPT_USERAGENT => "realtime-pubsubhubbub/1.0");

        $ch = curl_init();
        curl_setopt_array($ch, $options);
        curl_exec($ch);
        $info = curl_getinfo($ch);
        curl_close($ch);

        if ($info['http_code'] == 204) {
            return true;
        }
        return false;
    }

    public function subscribe($topic_url, $callback_url, $token=false, $lease=false) {
        if (!isset($topic_url)) {
            // nothing to do
            return;
        }
        if (!preg_match("|^https?://|i", $topic_url)) {
            throw new Exception('Invalid URL: ' . $topic_url);
        }

        $this->param('hub.topic', $topic_url);
        $this->param('hub.mode', 'subscribe');
        $this->param('hub.callback', $callback_url);
        $this->param('hub.verify', 'sync');

        if ($token) {
            $this->param("hub.verify_token", $token);
        }
        if ($lease) {
            $this->param("hub.lease_seconds", $lease);
        }

        return $this->_post();
    }

    public function publish($topic_url) {
        if (!isset($topic_url)) {
            // nothing to do
            return;
        }
        if (!preg_match("|https?://|i", $topic_url)) {
            throw new Exception('Invalid URL: ' . $topic_url);
        }
        $this->param('hub.url', $topic_url);
        $this->param('hub.mode', 'publish');
        return $this->_post();
    }
}
