// -*- indent-tabs-mode: nil;  -*-

var Home = {
    trends: [],                 // what is trending on twitter?
    twitter_since_id: 0,        // keep track of the twitter entries we've seen
    live_images_since_id: 0, // keep track of the twitter images we've seen
    friendfeed_entries: []      // keep track of the FF entries we've seen
};

Home.init = function() {
	// load the trending topics from twitter
    Home.appendJS('http://search.twitter.com/trends.json?callback=Home.catchTrends');
    Home.getFriendFeedUpdates();
    Home.getLiveImages();
};

Home.appendJS = function(url) {
    url += "&" + Math.random(); // ensure the browser doesn't cache this
    var scr = document.createElement('script');
    scr.setAttribute('src', url);
    scr.setAttribute('type', 'text/javascript');
    document.getElementsByTagName('head').item(0).appendChild(scr);
};


Home.catchTrends = function(data) {
    for(var i = 0; i < data.trends.length; ++i)
        Home.trends.push(data.trends[i].name);
    Home.getTrendingTweets();
};

Home.getTrendingTweets = function() {

    var url = 'http://search.twitter.com/search.json?' +
                  'callback=Home.catchTweets&q=';

    // randomize the array
    Home.trends = Home.trends.sort(function() {
                                       return (Math.round(Math.random()) - 0.5);
                                   });

    // search on five trending topics
    url += encodeURIComponent(Home.trends.slice(0,5).join(" OR "));
    url += "&since_id=" + Home.twitter_since_id;
    Home.appendJS(url);
    setTimeout(Home.getTrendingTweets, 7000);
};


Home.catchFriendFeedUpdates = function(data) {
    var cnt = 0;

    for(var i = data.entries.length - 1; i > -1; --i) {
        var entry = data.entries[i];

        if(Home.friendfeed_entries.indexOf(entry.id) == -1) {
            Home.friendfeed_entries.push(entry.id);
            Home.addLine('friendfeed-updates', { 'username': entry.from.name,
                                                 'html': entry.body
                                               });
            cnt++;
        }
    }

    if(cnt)
        Home.updateTitle('friendfeed-title', cnt);

};

Home.catchTweets = function(data) {
    var doc = document;
    var trendingDiv = doc.getElementById('trending-on-twitter');

    Home.twitter_since_id = data.max_id;

    for(var i = data.results.length - 1; i > -1; --i) {
        var tweet = data.results[i];
        Home.addLine('trending-on-twitter', { 'username': tweet.from_user,
                                              'html': tweet.text,
                                              'profile_image': tweet.profile_image_url
                                            });
    }

    if(data.results.length)
        Home.updateTitle('trending-title', data.results.length);

};

Home.catchLiveImages = function(data) {

    // setup the regexp to be used in the loop
    var image_regexp = new RegExp("http://(yfrog\.com|twitpic\.com)/([0-9a-zA-Z]+)");

    // loop through all the tweets returned by Home.getLiveImages
    for(var i = data.results.length - 1; i > -1; --i) {
        var tweet = data.results[i];

        // run the regular expression on the tweet
        var matches = image_regexp.exec(tweet.text);

        if(matches) {
            console.log(matches);
            // yfrog.com or twitpic.com
            var domain = matches[1];
            // the image ID to be used in the URL
            var image_id = matches[2];

            // setup the thumbnail URL for the different services
            var image_url = '';
            if (domain == 'yfrog.com')
                image_url = 'http://yfrog.com/' + image_id + '.th.jpg';
            if (domain == 'twitpic.com')
                image_url = 'http://twitpic.com/show/thumb/' + image_id;

            // generate an HTML string to show in the widget
            var image_str = '<a target="_blank" href="' + matches[0] + '">' +
                                '<img src="' + image_url + '" />'+
                                '</a>';

            // show the image (leave out the username and profile icon)
            Home.addLine('images-on-twitter', { 'html': image_str });
        }
    }

    if(data.results.length)
        Home.updateTitle('images-title', data.results.length);

    Home.live_images_since_id = data.max_id;

};

Home.getTrendingTweets = function() {

    var url = 'http://search.twitter.com/search.json?' +
                  'callback=Home.catchTweets&q=';

    // randomize the array
    Home.trends = Home.trends.sort(function() {
                                       return (Math.round(Math.random()) - 0.5);
                                   });

    // search on five trending topics
    url += encodeURIComponent(Home.trends.slice(0,5).join(" OR "));
    url += "&since_id=" + Home.twitter_since_id;
    Home.appendJS(url);
    setTimeout(Home.getTrendingTweets, 7000);
};

Home.getLiveImages = function() {
    var url = 'http://search.twitter.com/search.json?' +
                  'callback=Home.catchLiveImages&q=yfrog.com+OR+twitpic.com';
    url += "&since_id=" + Home.live_images_since_id;
    Home.appendJS(url);
    setTimeout(Home.getLiveImages, 7000);
};

Home.getFriendFeedUpdates = function() {
    url = "http://friendfeed-api.com/v2/feed/scobleizer/friends?" +
              "callback=Home.catchFriendFeedUpdates";
    Home.appendJS(url);
    setTimeout(Home.getFriendFeedUpdates, 5000);
};

Home.addLine = function(targetDivId, data) {
    var doc = document;
    var line = doc.createElement('div');

    line.setAttribute('class', 'line');

    if('profile_image' in data) {
        var img = doc.createElement('img');
        img.setAttribute('align', 'right');
        img.setAttribute('class', 'profile_image');
        img.setAttribute('src', data.profile_image);
        line.appendChild(img);
    }

    var p = doc.createElement('p');
    p.innerHTML = "";
    if('username' in data)
        p.innerHTML += '<strong>' + data.username + "</strong>: ";
    if('html' in data)
        p.innerHTML += data.html;

    line.appendChild(p);

    var targetDiv = doc.getElementById(targetDivId);
    targetDiv.insertBefore(line, targetDiv.firstChild); // wtf? (order or params)

    var dom_limit = 60;
    var elements = targetDiv.getElementsByClassName('line');
    for(var i = elements.length - 1; i >= dom_limit; --i)
        targetDiv.removeChild(elements[i]);

};

// based on Douglas Crockford's fade() function
// from JavaScript: The Good Parts
Home.highlight = function (node) {
};

Home.updateTitle = function(divId, number) {

    var titleDiv = document.getElementById(divId);

    // update and highlight the title
    titleDiv.innerHTML += ": " + number + " new results";

    // breifly highlight the title
    var highlightLevel = 1;
    var highlightStep = function () {
        var hex = highlightLevel.toString(16);
        titleDiv.style.backgroundColor = '#cccc' + hex + hex;
        if (highlightLevel < 12) {
            highlightLevel++;
            setTimeout(highlightStep, 50);
        }
    };
    highlightStep();

    // return it to the way it was
    setTimeout(function() {
                   titleDiv.innerHTML = titleDiv.innerHTML.replace(/\:.+/, '');
               }, 1500);

};

Home.init();