
import os, logging
import tornado.httpserver
import tornado.ioloop
import tornado.web
import uuid
import simplejson as json
from tornado.options import define, options
import threading

# Setup the queues
import Queue
tweetQueue = Queue.Queue(0)
 
# Define options that can be changed as we run this via the command line
define("port", default=8888, help="Run server on a specific port", type=int)

class MainHandler(tornado.web.RequestHandler):
    def get(self):
	logging.info("Request to MainHandler!")
	self.render("templates/main.html")

class Tweet(object):
    waiters = []
    cache = []
    cache_size = 200
    def wait_for_messages(self, callback, cursor=None):
        cls = Tweet
        if cursor:
            index = 0
            for i in xrange(len(cls.cache)):
                index = len(cls.cache) - i - 1
                if cls.cache[index]["id"] == cursor: break
            recent = cls.cache[index + 1:]
	    logging.info("%r recent tweets", len(recent))
            if recent:
                callback(recent)
                return
        cls.waiters.append(callback)

    def new_tweets(self, messages):
        cls = Tweet
        for callback in cls.waiters:
            try:
                callback(messages)
            except:
                logging.error("Error in waiter callback", exc_info=True)
        cls.waiters = []
        cls.cache.extend(messages)
        if len(cls.cache) > self.cache_size:
            cls.cache = cls.cache[-self.cache_size:]


class UpdateHandler(tornado.web.RequestHandler, Tweet):
    @tornado.web.asynchronous
    def post(self):
        cursor = self.get_argument("cursor", None)
        self.wait_for_messages(self.async_callback(self.on_new_tweets),
                               cursor=cursor)

    def on_new_tweets(self, tweets): 
        if not self.request.connection.stream.closed():
	    self.finish(dict(tweets=tweets))
	

# a consumer thread
import re
import simplejson as json
class TweetConsumerThread(threading.Thread):
    def run(self):
	hashtag = re.compile("(\#[0-9a-zA-Z]+)")
	ats = re.compile("(\@[0-9a-zA-Z]+)")
	links = re.compile("(http\:\/\/[^\ ]+)")
	retweets = re.compile("RT|via\ ")
	
        while True:
            tweet = tweetQueue.get()
            if tweet:
                t = json.loads(tweet)
		try:
                    stats = {}
                    stats['hashtags'] = hashtag.findall(t['text'])
                    stats['ats'] = ats.findall(t['text'])
                    stats['links'] = links.findall(t['text'])
                    stats['retweets'] = retweets.findall(t['text'])
		    # pack the message up
                    message = {
                        "id": str(uuid.uuid4()),
                        "stats": stats
                        }
                    message["html"] = "<div class=\"message\" id=\"m" + message["id"] + "\"><strong>" + t['user']['screen_name'] + ": </strong>" + t['text'] + "</div>"
		    # send the message out
		    Tweet().new_tweets([message])

		except Exception, e:
                    # ignore errors
		    pass



import urllib2
class TweetFirehose(threading.Thread): 
    def run(self):

	request = urllib2.Request('http://stream.twitter.com/1/statuses/sample.json')
 
	# add some basic authimport base64
	import base64 
	auth_basic = base64.encodestring('TWITTER_USERNAME:TWITTER_PASSWORD')[:-1]
	request.add_header('Authorization', 'Basic %s' % auth_basic)
 
	# open the connection
	firehose = urllib2.urlopen(request)
 
	for tweet in firehose:
	    if len(tweet) > 2:
		tweetQueue.put(tweet)
 
	firehose.close()



local_static_path = os.path.join(os.path.dirname(__file__), "static")

application = tornado.web.Application([
    (r"/", MainHandler),
    (r"/updates/?", UpdateHandler),
    
], static_path=local_static_path)


if __name__ == "__main__":
    # setup the twitter threads
    threads = []
    threads.append(TweetConsumerThread())
    threads.append(TweetFirehose())
 
    # for each thread, set daemon mode to true and start them up
    for t in threads:
        t.daemon = True
        t.start()

    http_server = tornado.httpserver.HTTPServer(application)
    tornado.options.parse_command_line()
    http_server.listen(options.port)
    tornado.ioloop.IOLoop.instance().start()
