import os, logging
import tornado.httpserver
import tornado.ioloop
import tornado.web
import uuid
import simplejson as json
from tornado.options import define, options


# Define options that can be changed as we run this via the command line
define("port", default=8888, help="Run server on a specific port", type=int)


class Chat(object):
    users = []
    listeners = []

    def add_listener(self, callback, user_id=None):
        cls = Chat
        data = {}
        logging.error("%s => %s" % (user_id, callback.__name__))
        data['user_id'] = user_id
        data['callback'] = callback
        cls.listeners.append(data)
        
    def add_user(self, user_id, user_name):
        self.users.append(dict(user_id=user_id, user_name=user_name))
        # let every user know about the new user
        message = {}
        message['type'] = 'login'
        message['data'] = dict(user_id=user_id, user_name=user_name)
        cls = Chat
        for data in cls.listeners:
            if user_id != data['user_id']:
                logging.info("Looking for callback for uesr_id: %s ", data['user_id'])
                callback = data['callback']
                try:
                    callback(message)
                except:
                    logging.error("Error in listeners callback", exc_info=True)
        # clear the thing
        cls.listeners = []

    def run_callbacks(self, message, user_id=None, except_user_id=None):
        cls = Chat
        tmp_listeners = cls.listeners
        cls.listeners = []

        for data in cls.listeners:

            run_callback = False
            
            # if we're sending it to all users, 
            if user_id == None: 
                run_callback = True
            # or this user
            elif user_id == data['user_id']:
                run_callback = True
                
            # if we're specifically NOT sending it to someone...
            if except_user_id == data['user_id']:
                run_callback = False
            
            if run_callback:
                logging.info("Looking for callback for uesr_id: %s ", data['user_id'])
                callback = data['callback']
                try:
                    callback(message)
                except:
                    logging.error("Error in listeners callback", exc_info=True)
            else:
                cls.listeners.append(data)

            
class MainHandler(tornado.web.RequestHandler):
    def get(self):
	self.render("templates/chat-main.html")

class LoginHandler(tornado.web.RequestHandler, Chat):
    def post(self):
        user_name = self.get_argument('username')
        user_id = str(uuid.uuid4())
        logging.info("LOGIN: %s - %s" % (user_name, user_id))
        self.add_user(user_id, user_name)
        self.finish(dict(user_id=user_id, user_name=user_name, users=self.users))


# handle a new "send" chat request from a user
class SendHandler(tornado.web.RequestHandler, Chat):
    def post(self):
        to_user_id = self.get_argument('to_user_id')
        from_user_id = self.get_argument('from_user_id')
        # let every user know about the new user
        message = {}
        message['type'] = 'message'
        message['data'] = dict(from_user_id=from_user_id, to_user_id=to_user_id, text=self.get_argument('text'))
        cls = Chat

        logging.info("FOUND %r listeners", len(cls.listeners))
        tmp_listeners = cls.listeners
        cls.listeners = []

        for data in tmp_listeners:
            logging.info("SEND? %s == %s ?" % (to_user_id, data['user_id']))
            try:
                if data['user_id'] == to_user_id:
                    callback = data['callback']
                    callback(message)
                else:
                    cls.listeners.append(data)

            except:
                self.finish(dict(send_status='error', to_user_id=to_user_id))

        message['type'] = 'sent'
        self.finish(message)


class TypingHandler(tornado.web.RequestHandler, Chat):
    def post(self):
        to_user_id = self.get_argument('to_user_id')
        from_user_id = self.get_argument('from_user_id')
        # let every user know about the new user
        message = {}
        message['type'] = 'typing'
        message['data'] = dict(from_user_id=from_user_id, to_user_id=to_user_id)
        cls = Chat

        logging.info("FOUND %r listeners", len(cls.listeners))
        tmp_listeners = cls.listeners
        cls.listeners = []

        for data in tmp_listeners:
            logging.info("SEND? %s == %s ?" % (to_user_id, data['user_id']))
            try:
                if data['user_id'] == to_user_id:
                    callback = data['callback']
                    callback(message)
                else:
                    cls.listeners.append(data)

            except:
                self.finish(dict(send_status='error', to_user_id=to_user_id))

        message['type'] = 'recv'
        self.finish(message)


# wait for updates and return them when available
class UpdateHandler(tornado.web.RequestHandler, Chat):
    @tornado.web.asynchronous
    def post(self):
        user_id = self.get_argument('user_id')
        self.add_listener(self.async_callback(self.handle_updates),
                          user_id=user_id)

    def handle_updates(self, update): 
        if not self.request.connection.stream.closed():
            self.finish(update)



local_static_path = os.path.join(os.path.dirname(__file__), "static")

application = tornado.web.Application([
    (r"/", MainHandler),
    (r"/login", LoginHandler),
    (r"/updates", UpdateHandler),
    (r"/send", SendHandler),
    (r"/typing", TypingHandler),
], static_path=local_static_path)

if __name__ == "__main__":
    http_server = tornado.httpserver.HTTPServer(application)
    tornado.options.parse_command_line()
    http_server.listen(options.port)
    tornado.ioloop.IOLoop.instance().start()
