import wsgiref.handlers
import os, logging

from google.appengine.ext.webapp import template 
from google.appengine.ext import webapp, db
from google.appengine.api import xmpp, users, urlfetch
from google.appengine.ext.webapp.util import login_required

from django.utils import simplejson as json

class BaseHandler(webapp.RequestHandler):
    # the jabber id of the server
    server_jid = os.environ['APPLICATION_ID'] + '@appspot.com'
    # the server URL
    server_url = "http://%s.appspot.com" % os.environ['APPLICATION_ID']

class IMUser(db.Model):
    account = db.UserProperty()
    email = db.EmailProperty()

    def is_authenticated(self, email):
        # remove the "resource" string to reveal the email address by itself
        if email.find('/') != -1:
            email = email[:email.find('/')]
        im_user = IMUser.gql('WHERE email = :1', email).get()
        if im_user: 
            return True
        else:
            return False
        

class MainHandler(BaseHandler): 
    @login_required
    def get(self):
        user = users.get_current_user()
        logging.info("Got request for MainHandler")

        # get the IMUser if they've been here before
        u = IMUser.gql('WHERE email = :1', user.email()).get()
        if not u:
            # if it's their first time here, store it in the cloud
            u = IMUser(email=user.email(), account=user)
            u.put()

        self.response.out.write('Hello, %s!' % user.nickname())

def get_weather_for_zipcode(zipcode):
    yql = "select item from weather.forecast where location = %s" % zipcode
    yql = yql.replace(" ", "%20")
    url = "http://query.yahooapis.com/v1/public/yql?format=json&q=%s" % yql
    response = urlfetch.fetch(url)
    data = json.loads(response.content)
    try: 
        return data['query']['results']['channel']['item']['condition']
    except: 
        return None

class APISendHandler(BaseHandler):
    def post(self):
        # pull out who the message is to, and what they want to say
        to = self.request.get('to')
        body = self.request.get('body')

        logging.info("to: " + to)
        logging.info("body: " + body)

        if not IMUser().is_authenticated(to):
            self.response.out.write(json.dumps({'status': 'fail', 
                                                'mesg': 'User not authenticated.'}))
            return

        # check to see if the "password" matches
        if self.request.get('secret') == 's3cret-c0de':
            # if so, send the message and write a response
            xmpp.send_message(to, body)
            logging.info("to: %s, body: %s" % (to, body))
            self.response.out.write(json.dumps({'status': 'ok', 'mesg': 'Sent.'}))
                                    
        # if the password is wrong, don't send any instant message
        else:
            self.response.out.write(json.dumps({'status': 'fail', 
                                                'mesg': 'Wrong secret code.'}))

class XMPPHandler(BaseHandler):
    def post(self):
        message = xmpp.Message(self.request.POST)
        logging.info("XMMP sender: %s - body: %s" % (message.sender, message.body))

        # The command from the user
        cmd = message.body.split(' ')[0].lower()
        # the rest of the message
        body = message.body[len(cmd) + 1:]

        # get the IMUser object
        if not IMUser().is_authenticated(message.sender):
            message.reply("Please register, it's painless: %s" % self.server_url)
            return

        # echo the data back to the client
        if cmd == 'echo':
            message.reply(body)

        # convert string to rot13 and repeat it back to them
        elif cmd == 'rot13':
            message.reply(body.encode('rot13'))

        # add up all the numbers
        elif cmd == 'sum':
            numbers = body.split(" ")
            total = 0
            # try to add the numbers together
            try:
                for x in numbers: 
                    total += int(x)
                text = "%s = %d" % (" + ".join(numbers), total)
            except:
                text = "Couldn't add these up. Invalid numbers?"

            message.reply(text)

        elif cmd == 'weather':
            weather = get_weather_for_zipcode(body)
            if weather:
                text = "Currently \"%s\" - temp %s" % (weather['text'],
                                                           weather['temp'])
            else:
                text = "Couldn't load weather? Invalid Location? Try again."

            message.reply(text)

        # we didn't get a command that we understand... complain
        else:
            message.reply("I don't understand '%s'" % cmd)

             
def main():
    application = webapp.WSGIApplication([('/', MainHandler),
                                          ('/_ah/xmpp/message/chat/', XMPPHandler),
                                          ('/api/send', APISendHandler)
                                          ],
                                         debug=True)
    wsgiref.handlers.CGIHandler().run(application)


if __name__ == '__main__':
    main()
