// -*- indent-tabs-mode: nil -*-
var Analytics = {
    uid: 0,
    start_time: 0,
    last_activity: 0,
    ping_url: "{{ ping_url }}",

    start: function() {
        Analytics.start_time = new Date().getTime();
        Analytics.last_activity = Analytics.start_time;

        // ensure we have a user ID
        Analytics.setupUID();

        // setup the event listeners
        var updateLastActivity = function() {
            Analytics.last_activity = new Date().getTime();
        };
        window.addEventListener('click',     updateLastActivity, false);
        window.addEventListener('scroll',    updateLastActivity, false);
        window.addEventListener('mousemove', updateLastActivity, false);
        window.addEventListener('keypress',  updateLastActivity, false);

        // when we're unloading the page
        var beforeUnload = function() {
            Analytics.last_activity = new Date().getTime();
            // ping again, but note that we're exiting
            Analytics.ping('exit');
        };

        // ensure that we get one last ping in before we unload the page
        window.addEventListener('beforeunload', beforeUnload, false);

        // start pinging.
        Analytics.ping();
    }
};

Analytics.setupUID = function() {
    // check for a cookie
    var cookie_name = "realtime_analytics";
    var cookies = document.cookie.split(/\; ?/g);
    for(var i = 0; i < cookies.length; ++i) {
        var name = cookies[i].split("=")[0];
        if (name == cookie_name) {
            Analytics.uid = cookies[i].split("=")[1];
            return;
        }
    }

    // if we're here, we need to generate and store a UID
    // generate the UID
    Analytics.uid = new Date().getTime().toString();
    Analytics.uid += "-" + Math.floor(Math.random() * 100000);

    // save it as a cookie
    var expire = new Date();
    expire.setDate(expire.getDate()+30); // 30 days in the FUTURE
    var cookie_data = cookie_name + "=" + escape(Analytics.uid) +
        ";expires=" + expire.toGMTString();
    document.cookie = cookie_data;
};

Analytics.ping = function(on_exit) {

    // was this method called because we were exiting the page?
    if ((typeof on_exit != 'undefined') && (on_exit == 'exit'))
        on_exit = 1;
    else
        on_exit = 0;

    var params =
        "&u=" + Analytics.uid +
        "&x=" + (typeof window.pageXOffset != 'undefined' ? window.pageXOffset : 0)  +
        "&y=" + (typeof window.pageYOffset != 'undefined' ? window.pageYOffset : 0)  +
        "&l=" + (new Date().getTime() - Analytics.last_activity)  +
        "&t=" + (new Date().getTime() - Analytics.start_time) +
        "&e=" + on_exit +
        "&dt=" + encodeURIComponent(document.title.slice(0, 50)) +
        "&r=" + encodeURIComponent(document.referrer ? document.referrer : '-direct-') +
        "&*=" + Math.random();

    var scr = document.createElement('script');
    scr.setAttribute('type', 'text/javascript');
    scr.setAttribute('src', Analytics.ping_url + '?' + params);
    document.getElementsByTagName('head')[0].appendChild(scr);

    // setup a loop to ping again in 3 seconds
    window.setTimeout(Analytics.ping, 3000);
};

