# -*- indent-tabs-mode: nil -*-
import os, logging
import tornado.httpserver
import tornado.ioloop
import tornado.web
import simplejson as json
import time
import re
from tornado.options import define, options
from tornado.escape import url_escape
import urllib


# Define options that can be changed as we run this via the command line
define("port", default=8888, help="Run server on a specific port", type=int)

class Analytics(object): 
    users = {}
    pages = {}
    data = {}

    SMS_alert_mobile_number = '+YOURNUMBERHERE'
    SMS_alert_current_users = 15
    SMS_api_base_url = 'http://instant-messaging.appspot.com/sms/send/via/textmark'
    SMS_last_send = 0
    SMS_send_interval = 600 # send an SMS no more than every ten minutes

    # Number of seconds for which a session is live.
    # After this time, they're considered idle.
    # the user must continue to ping
    live_session_time = 30  
                            
    # If the ping timeout passes, the user has left the page.
    ping_timeout_time = 30


    def __init__(self):
        logging.info("Init")

    def remove_user_url(self, uid, url):
        if uid not in self.users:
            return
        # EVENT: a user is gone!
        print "Removing a user: %s - %s" % (uid, url)
        try:
            del self.users[uid][url]
        except Exception, e:
            logging.error(e)
            logging.error(self.users)
            logging.info("The user was probably remove twice")
        
    def update_data(self):
        # remove stale users
        active_user_count = 0
        self.pages = {}
        for x in self.users.keys():
            for url in self.users[x].keys():
                last_ping_in_seconds = int(self.users[x][url]['last_ping'])
                last_activity = int(self.users[x][url]['last_activity']) / 1000
                if url not in self.pages:
                    self.pages[url] = 0
                self.pages[url] += 1

                if last_activity < Analytics.live_session_time:
                    active_user_count += 1
                else:
                    self.users[x][url]['status'] = 'Idle'
                if last_ping_in_seconds < (time.time() - self.ping_timeout_time):
                    self.remove_user_url(x, url)
            if not len(self.users[x]):
                print "Removing: %s" % x
                del self.users[x]

#        logging.info("Active_User_Count: %d" , active_user_count)
        if active_user_count > Analytics.SMS_alert_current_users:
            if Analytics.SMS_last_send < (time.time() - Analytics.SMS_send_interval):
                Analytics.SMS_last_send = time.time()
                url = Analytics.SMS_api_base_url
                url += '?mobile_number=' + url_escape(Analytics.SMS_alert_mobile_number)
                url += '&body=' + url_escape("Currently %d users on the site! (Huzzah!)" % (len(self.users)))

                f = urllib.urlopen(url)
                logging.info(f.read())
            
        
        for x in self.data.keys():
            tmp_data = self.data[x]
            self.data[x] = []
            for d in tmp_data:
                if time.time() > d['expire_time']:
                    print "**** remove %s => %s ******" % (x['type'], x['value'])
                else:
                    self.data[x].append(d)
            
    def data_ping(self, data_type, value, expire_time):
        if not data_type in self.data:
            self.data[data_type] = []
        
        self.data[data_type].append({'type': data_type, 
                                     'value': value,
                                     'action_time': time.strftime("%I:%M:%S %p", time.localtime()),
                                     'expire_time': expire_time})
        
        
    def user_ping(self, uid, url=0, user_agent=0, domain=0, last_activity=0, time_on_page=0, x=0, y=0, title=0, referrer=0):
        if uid not in self.users:
            print "Got a new user %s" % user_agent
            self.users[uid] = {}
            # EVENT: We have a new user!
            pass

        self.users[uid][url] = {'url': url,
                                'user_agent': user_agent,
                                'title': title,
                                'domain': domain,
                                'referrer': referrer,
                                'x': x,
                                'y': y,
                                'time_on_page': time_on_page,
                                'last_activity': last_activity,
                                'last_ping': time.time(),
                                'status': 'Active'
                                }
        self.update_data()

        

class PingHandler(tornado.web.RequestHandler): 
    def get(self):

        # setup some local convenience variables for later
        uid = self.get_argument('u')
        url = self.request.headers['Referer']

        # build the data object to pass to the user_ping method
        data = {
            'uid': uid,
            'url': url,
            'user_agent': self.request.headers['User-Agent'],
            'referrer': self.get_argument('r', '-direct-'),
            'time_on_page': self.get_argument('t'),
            'last_activity': self.get_argument('l'),
            'x': self.get_argument('x'),
            'y': self.get_argument('y'),
            'title': self.get_argument('dt')
            };

        self.application.analytics.user_ping(data)


class PingHandler(tornado.web.RequestHandler): 
    def get(self, domain):

        analytics = self.application.analytics

        uid = self.get_argument('u')
        url = self.request.headers['Referer']

        user_agent = self.request.headers['User-Agent']

        ping_on_exit = self.get_argument('e')
        if ping_on_exit is not None and ping_on_exit == "1":
            analytics.remove_user_url(uid, url)
            return

        data = {
            'url': url,
            'user_agent': user_agent,
            'domain': domain,
            'referrer': self.get_argument('r', '-direct-'),
            'time_on_page': self.get_argument('t'),
            'last_activity': self.get_argument('l'),
            'x': self.get_argument('x'),
            'y': self.get_argument('y'),
            'title': self.get_argument('dt')
            };

        logging.info(user_agent)
        analytics.user_ping(uid, **data)
                 

class CustomHandler(tornado.web.RequestHandler):
    def get(self):
        self.respond()
    def post(self):
        self.respond()

    def respond(self):
        analytics = self.application.analytics
        data_type = self.get_argument('type')
        value = self.get_argument('value')
        expires = self.get_argument('expires_in_seconds')
        analytics.data_ping(data_type, value, time.time() + int(expires))
        self.write("OK")
        


class GetStatsHandler(tornado.web.RequestHandler):
    def post(self):
        a = self.application.analytics
        # remove some stale users
        a.update_data()
        sortable_pages = []
        for x in a.pages:
            sortable_pages.append({'url': x, 'count': a.pages[x]});

        def compare(a, b):
            return cmp(b['count'], a['count'])
        sortable_pages.sort(compare)
        self.write(dict(users=a.users, data=a.data, pages=sortable_pages[:5]))


class JSHandler(tornado.web.RequestHandler):
    def get(self):
        ping_url = self.request.protocol + "://" + self.request.host + "/ping"
        self.set_header("Content-Type", "text/javascript")
        self.render("realtime-analytics.js", ping_url=ping_url)

class HomeHandler(tornado.web.RequestHandler):
    def get(self):
        self.render("home.html")

class Application(tornado.web.Application):
    def __init__(self):

        self.analytics = Analytics()

        handlers = [
            (r"/ping/(.+)?", PingHandler),
            (r"/js/?", JSHandler),
            (r"/", HomeHandler),
            (r"/get/stats/?", GetStatsHandler),
            (r"/custom/?", CustomHandler),
            ]

        settings = dict(
            static_path=os.path.join(os.path.dirname(__file__), "static"),
        )
        tornado.web.Application.__init__(self, handlers, **settings)



if __name__ == "__main__":
    tornado.options.parse_command_line()
    http_server = tornado.httpserver.HTTPServer(Application())
    http_server.listen(options.port)
    tornado.ioloop.IOLoop.instance().start()
