// ask google to load the jquery libraries
google.load("jquery", "1");

// when the library is loaded, get
google.setOnLoadCallback(
    function() {
        $(document).ready(A.poll);
    }
);

// create the main client side analytics object
var A = {
    poll: function() {
        // poll the server using ajax
        $.ajax({
            url: "/get/stats",
            type: "POST",
            dataType: "json",
            success: A.catch_new_data
        });
    }
};


A.display_users = function(users) {

	// grab the HTML divs and clear them out
    var active_list = $('#active_users');
    var idle_list = $('#idle_users');
    idle_list.html('');
    active_list.html('');

	// loop through the users object
    for (var uid in users) {

        var user = users[uid];
        // each user can be looking at multiple URLs, loop through them
        for (var url in user) {
            u = user[url];

			// create a list item to add to the ul object
            var li = $('<li>');

			// create a link object to display the page the user is viewing
            var a = $('<A>');
            a.attr('href', u.url);
            a.attr('title', u.user_agent);
            a.html(u.title);
            li.append(a);

            li.append($('<span>').html(' / '));

			// what domain/website is the user looking at
            var dom = $('<a>');
            dom.attr('href', 'http://' + u.domain);
            dom.attr('class', 'domain');
            dom.html(u.domain);
            li.append(dom);

			// create a DIV to hold some other data
            var data = $('<div>');
            data.attr('class', 'data');
            li.append(data);

			// display the referrer
            if(u.referrer) {
                var ref = $('<span>');
                ref.attr('class', 'referrer');
                ref.html('<a href="' + u.referrer + '">' + u.referrer.slice(0, 50) + "</a>");
                data.append(ref);
            }

			// display the last activity and current scroll position
            var t = u.last_activity / 1000;
            if (t > 60) t = Math.round(t / 60) + ' minute(s)';
            else t += " seconds";
            var time = $('<span>');
            time.html('Idle Time: ' + t + ' | Scroll x: ' + u.x + ' y: ' + u.y);
            data.append(time);

			// if the user is idle, add them to the idle list
            if (u.status == 'Idle')
                idle_list.append(li);
			// otherwise to the active list
            else
                active_list.append(li);

        }
    }
};

A.display_custom = function(data) {
    var div = $('#custom_data');
    div.html(""); // clear it

    if(typeof data == 'object') {
        for (var data_type in data) {

            var h2 = $('<h2>');
            h2.html(data_type);
            div.append(h2);

            var ul = $('<ul>');
			console.log(data_type);
			console.log(data[data_type]);
            for (var i = 0; i < data[data_type].length; ++i) {
                var d = data[data_type][i];
                var li = $('<li>');
                li.html(d.value + " (" + d.action_time + ')');
                ul.append(li);
            }
            div.append(ul);
        }
    }

};


A.display_pages = function(data) {

	// grab the pages list and clear it
    var ul = $('#pages');
    ul.html('');

	// loop through the pages array
    for(var i = 0; i < data.length; ++i) {

        var url = data[i].url;
        var cnt = data[i].count;
        var li = $('<li>');
        var a =  $('<a>');
        a.attr('href', url);
		// remove the "http://" from the URL when displaying it
        a.html(url.slice(0, 50).replace(/^https?\:\/\//, ''));
        li.append(a);

        var s = $('<span>');
        s.html(" (" + cnt + ')');
        li.append(s);

        ul.append(li);
    }
};

A.catch_new_data = function(response) {
    if(response) {
        A.display_users(response.users);
        A.display_custom(response.data);
        A.display_pages(response.pages);
    }

    setTimeout(A.poll, 1000);
};