# standard python library stuff 
import wsgiref.handlers
import os, logging, random, urllib, cgi, re
from datetime import datetime

# app engine imports
from google.appengine.ext.webapp import template 
from google.appengine.ext import webapp, db
from google.appengine.api import xmpp, users, urlfetch
from google.appengine.ext.webapp.util import login_required
from django.utils import simplejson as json


# import all models:
from models import *

class MainHandler(webapp.RequestHandler):
    def get(self):

        # this is really bad, we're just looking to see if it's mobile safari
        # android, iphone, etc
        if self.request.headers['User-Agent'].find('Mobile') == -1:
            logging.info("NOT MOBILE!!!")
            self.redirect("/wtf")
            return

        # load the current user from google
        current_user = users.get_current_user()

        # the file location of the logged out template
        template_path = os.path.join(os.path.dirname(__file__), 
                                     'templates/main-logged-out.html')

        # setup the basic template variables
        template_values = {
            "user": current_user,
            "login_url": users.create_login_url('/sync/user'),
            "logout_url": users.create_logout_url('/')
            }


        if current_user:
            template_path = os.path.join(os.path.dirname(__file__), 
                                         'templates/main.html')

            # load the UserInfo object for this user
            ui = UserInfo().by_user(current_user)

            # add UserInfo data to the template
            template_values["allow_sms"] = ui.allow_sms
            template_values["mobile_key"] = ui.mobile_key
            template_values["mobile_number"] = ui.mobile_number
            template_values["mobile_short_code"] = Textmark.short_code
            template_values["mobile_keyword"] = Textmark.keyword

        # render the template
        self.response.out.write(template.render(template_path, template_values))


class InstructionsHandler(webapp.RequestHandler):
    def get(self):
        path = os.path.join(os.path.dirname(__file__), 'templates/instructions.html')
        self.response.out.write(template.render(path, {}))

class SyncUserHandler(webapp.RequestHandler):
    def get(self):
        UserInfo().sync(users.get_current_user())
        self.redirect('/instructions')
        
class CoughActionHandler(webapp.RequestHandler):
    def post(self):
        lat = float(self.request.get('lat'))
        lon = float(self.request.get('lon'))

        current_user = users.get_current_user()
        user_info = UserInfo().by_current_user()

        # get the default disease for this user
        d = Disease().get_default_disease_for_user(current_user)

        # create a new germ (not saved right away)
        new_germ = Germ(user=current_user, 
                        disease=d,
                        location=db.GeoPt(lat, lon))

        target_germ_id = self.request.get('target_germ_id')

        if target_germ_id:
            enemy = db.get(target_germ_id)

            logging.info(enemy)

            # if our new germ defeats the old germ
            # delete the enemy germ and continue on to create a new one
            if new_germ.would_defeat(enemy):
                CommandCenter().delete_germ(enemy)
                enemy.delete()

            # we couldn't defeat the other germ!!!
            else:
                message = "You didn't defeat this germ. Spread your disease elsewhere and try again."
                self.response.out.write(json.dumps({'message': message, 'germs': []}))
                # get out of here.
                return
        
        germs = Germ().get_germs_near_point(lat, lon,
                                            max_distance_in_meters=1000) 
        current_user = users.get_current_user()
        for g in germs:
            distance = GameUtils().distance_in_miles(lat, lon, g.location.lat, g.location.lon)
            logging.info("COUGH::DIST: %f, spread of germ: %f" % (distance, g.spread))
            if distance < g.spread: 
                if g.user == current_user:
                    logging.info("YOU HAVE ONE HERE already adding to it!")
                    g.invigorate()
                    germ = g.format_for_json(current_user)
                    germ['color'] = Game().default_color_friendly
                    self.response.out.write(json.dumps(germ))
                    return
                # else:
                #     # it was probably just created. They'll fight it out automatically
                #     logging.info("A GERM IS IN THE CURRENT SPACE!")
                    
        # OK. we can create this new germ
        logging.info("Putting the data for the new germ")
        new_germ.update_location()
        new_germ.put()

        CommandCenter().add_germ(new_germ)
        CommandCenter().update()
        
        # update the disease
        d.strength += new_germ.strength
        logging.info("increasing disease strength")
#        logging.info("INCREASING DISEASE STRENGTH")
        d.put()

        germ = new_germ.format_for_json(current_user)
        germ['color'] = Game().default_color_friendly
        self.response.out.write(json.dumps({'message': '', 'germs': [germ]}))



class GetNearbyGermsHandler(webapp.RequestHandler):
    def post(self):
        lat = float(self.request.get('lat'))
        lon = float(self.request.get('lon'))
        germs = Germ().get_germs_near_point(lat, lon,
                                            max_distance_in_meters=1609*3) # a few miles
        output = []
        current_user = users.get_current_user()
        for germ in germs:
            # the same germ in JSON format
            g = germ.format_for_json(current_user)

            # assume this is an enemy
            g['color'] = Game().default_color_enemy 

            if germ.user == current_user:
                g['color'] = Game().default_color_friendly
            else:
                distance = GameUtils().distance_in_miles(lat, 
                                                        lon, 
                                                        germ.location.lat, 
                                                        germ.location.lon)

                if distance < germ.spread: 
                    # FIXME: if the user was threatened log it and ensure they reach
                    g['color'] = Game().default_color_threat
                    g['threat'] = True

                    # this user is threated by this. 
                    # log it so we can force them to do something
                    UserThreats().create(current_user, germ)

            output.append(g)

        if len(germs) == 1:
            message = '1 germ nearby!'
        else:
            message = '%d germs nearby!' % len(germs)
        message = ''
        self.response.out.write(json.dumps({'message': message,
                                            'germs': output}))

class CoverActionHandler(webapp.RequestHandler):
    def post(self):

        target_germ_id = self.request.get('target_germ_id')
        current_user = users.get_current_user()

        if target_germ_id:

            enemy = db.get(target_germ_id)
            logging.info("COVER TARGET_GERM: %s" % target_germ_id)

            message = ""

            current_user_info = UserInfo().by_user(current_user)

            # your score decreases... because that was kinda weak.
            current_user_info.update_score(-1)

            UserThreats().remove_threat(current_user, enemy)

            # They didn't successfully cover UP!
            # deduct points, strength the germ, and 
            # award points to your adversary
            if random.randint(1,10) == 2:
                message = "Wow! You didn't cover up successfully! Your score and disease strength BOTH decrease. Nuts!"
                # you covered up, your disease strength increases
                disease = Disease().get_default_disease_for_user(current_user)
                disease.strength -= Game().default_disease_strength_increment

                # your enemy gets points for defeating you!
                enemy_user_info = UserInfo().by_user(enemy.user)
                enemy_user_info.update_score(+3)
                enemy_disease = Disease().get_default_disease_for_user(enemy.user)
                enemy_disease.strength += Game().default_disease_strength_increment
                enemy_disease.put()

            else:
                message = "You covered up successfully! Your disease is stronger!"
                
                # you covered up, your disease strength increases
                disease = Disease().get_default_disease_for_user(current_user)
                disease.strength += Game().default_disease_strength_increment

            disease.put()
            current_user_info.put()

            self.response.out.write(json.dumps({'message': message,
                                                'germs': []}))

        

class MessageToGermHostHandler(webapp.RequestHandler):
    def post(self):
        g = Germ.get(self.request.get('target_germ_id'))

        # a couple of things happened here:
        # by sending this message, the current user has instigated
        # a threat against the enemy user.
        # if the enemy responds in two minutes
        ut = UserThreats().create(g.user, g.key(), 
                                  expires_in_minutes=2,
                                  challenge_user=users.get_current_user(),
                                  threat_type='challenge')
        
        if ut:
            mesg = self.request.get('msg')
            mesg += "\n\nThis is a direct threat! Protect yourself."
            Messenger().send(from_user=users.get_current_user(), 
                             to_user=g.user,
                             body=mesg, 
                             response_key=ut.response_key)


class CronSpreadGermsHandler(webapp.RequestHandler):
    def get(self):
        Germ().auto_spread()

class CronFightGermsHandler(webapp.RequestHandler):
    def get(self):
        Germ().auto_fight()

class CronThreatHandler(webapp.RequestHandler):
    def get(self):
        UserThreats().update_threats()


class XMPPHandler(webapp.RequestHandler):
    def post(self):
        message = xmpp.Message(self.request.POST)
        
        logging.info("XMMP sender: %s - body: %s" % (message.sender, message.body))
        
        # clean up the resource name
        email = message.sender
        if email.find('/') != -1:
            email = email[:email.find('/')]

        sender = users.User(email)
        body = message.body.strip()

        if body.lower() in ['ok', 'start', 'stop']:
            ui = UserInfo().by_user(sender)
            if body.lower() == 'stop':
                ui.allow_xmpp = False
                response = "OK, this is your last message until you send me 'ok' (which will tell me you want to get started again."
            else:
                ui.allow_xmpp = True
                response = "OK, you're all signed up to receive instant messages. Send 'stop' if you ever want to quit."
            ui.put()
            message.reply(response)
            return


        try:
            results = re.findall("@([0-9]+)\ (.+)", body)
            logging.info(results)
            if len(results) == 1:
                code, msg = results[0]
                logging.info("Doing: %s" % sender)
                response = UserThreats().respond_to_threat(sender, int(code), msg)
                logging.info("done")
                if response:
                    logging.info("response: %s", response)
                    message.reply(response)
                else:
                    message.reply("Message delivered, we'll see what happens.")
                return
        except Exception, e:
            pass

        message.reply("I had trouble dealing with that... sorry :(")




class SMSIncomingHandler(webapp.RequestHandler):

    # handle the GET requests
    def get(self):
        sms = Textmark()

        # start the response
        self.response.headers['Content-Type'] = 'text/plain'

        # prepare some HTTP parameters as variables
        uid = self.request.get('uid')
        body = self.request.get('body').strip()
        clean_body = body.strip()
        
        ui = UserInfo().by_mobile_number(uid)

        # great we found this user.
        if ui:
            if clean_body == 'stop':
                ui.allow_sms = False
                ui.put()
                self.response.out.write("OK, SMS has been disabled. You may re-JOIN whenever you want.")
                return

            results = re.findall("@([0-9]+)\ (.+)", clean_body)                
            if len(results) == 1:
                code, msg = results[0]
                logging.info("Doing: %s" % ui.user)
                response = UserThreats().respond_to_threat(ui.user, int(code), msg)
                self.response.out.write(response)
            else:
                self.response.out.write("I'm not sure what you wanted me to do.")

        # we didn't find this user by UID.
        else:
            # perhaps they're trying to sign up?
            logging.info("Clean body: '%s'" % clean_body)
            ui = UserInfo().by_mobile_key(clean_body)
            if ui:
                # check
                ui.allow_sms = True
                ui.mobile_number = uid
                ui.put()
                self.response.out.write("OK, SMS is enabled. Good choice.")
            else: 
                self.response.out.write("Hrm. I couldn't find that mobile key. Try again?")
            return
            

class WTFHandler(webapp.RequestHandler):
    def get(self):
        path = os.path.join(os.path.dirname(__file__), 'templates/wtf.html')
        self.response.out.write(template.render(path, {}))


class FeedHandler(webapp.RequestHandler):
    def get(self):

        germs = CommandCenter().get_germs()
        output = []
        for x in germs:
            try:
                germs[x]['json'] = json.dumps(germs[x])
                output.append(germs[x])
            except:
                pass
    
        template_values = {
            'germs': output,
            'url': 'http://' + os.environ["HTTP_HOST"],
            'updated': GameUtils().date_rfcformat(datetime.now()),
            'title': 'iPandemic'
            }
        logging.info(template_values)
        path = os.path.join(os.path.dirname(__file__), 'templates/atom.xml')

        self.response.headers['Content-Type'] = 'application/atom+xml'
        self.response.out.write(template.render(path, template_values))


# class ControlCenterHandler(webapp.RequestHandler):
#     def get(self):
#         path = os.path.join(os.path.dirname(__file__), 'templates/control-center.html')
#         template_values = {
#             "user": users.get_current_user(),
#             "login_url": users.create_login_url('/sync/user'),
#             "logout_url": users.create_logout_url('/')
#             }
#         self.response.out.write(template.render(path, template_values))


def main():
    application = webapp.WSGIApplication([('/', MainHandler),
                                          ('/cough/?', CoughActionHandler),
                                          ('/cover/?', CoverActionHandler),
                                          ('/message/to/germ/host/?', MessageToGermHostHandler),
                                          ('/sync/user/?', SyncUserHandler),
                                          ('/cron/update/threats/?', CronThreatHandler),
                                          ('/get/nearby/germs/?', GetNearbyGermsHandler),
                                          ('/_ah/xmpp/message/chat/?', XMPPHandler),
                                          ('/catch/sms/?', SMSIncomingHandler),
                                          ('/cron/spread/germs/?', CronSpreadGermsHandler),
                                          ('/cron/fight/germs/?', CronFightGermsHandler),
                                          ('/feed/?', FeedHandler),
                                          # not needed
                                          ('/instructions/?', InstructionsHandler),
                                          ('/wtf/?', WTFHandler),

                                          ],
                                         debug=True)
    wsgiref.handlers.CGIHandler().run(application)


if __name__ == '__main__':
    main()
