// ask google to load the jquery libraries
google.load("jquery", "1");

var ipGeo = {
	map: false,
	default_lat: 40.730521,
	default_lon: -73.984337,
	points_in_circle: 10,
	current_germ_threat: ''
};

ipGeo.initialize = function(map_div_id) {
    if (GBrowserIsCompatible()) {
        ipGeo.map = new GMap2(document.getElementById(map_div_id));
        ipGeo.map.setCenter(new GLatLng(ipGeo.default_lat, ipGeo.default_lon), 13);
		ipGeo.map.setUIToDefault();

		// setup the map
		ipGeo.getLocation(function(position) {
							  var lat = position.coords.latitude;
							  var lon = position.coords.longitude;
							  ipGeo.map.setCenter(new GLatLng(lat, lon), 17);
							  ipGeo.showNearbyGerms(lat, lon);
						  }, function() {
							  alert("Your browser doesn't seem to support gelocation");
						  });
	}
	else
		alert("Google doesn't think your browser is compatible with their maps, this will be painful without them...");
};


ipGeo.fullscreenHide = function(html) {
	$('#fullscreen').html('');
	$('#fullscreen').hide();
};

ipGeo.fullscreenShow = function(html) {
	$('#fullscreen').css('display', 'block');
	var a  = $('<div>')
				 .click(ipGeo.fullscreenHide)
				 .attr('class', 'close')
				 .html('[close]');
	$('#fullscreen').append(a);
	$('#fullscreen').append(html);
};

ipGeo.getLocation = function(successCB, failCB) {
	if(navigator.geolocation && navigator.geolocation.getCurrentPosition)
		navigator.geolocation.getCurrentPosition(successCB, failCB, {maximumAge: 300000});

	else
		successCB({coords: {latitude: ipGeo.default_lat,
							longitude: ipGeo.default_lat}});
};

ipGeo.cover = function() {
	ipGeo.getLocation(function(position) {
						  $.post('/cover/',
								 { 'target_germ_id': ipGeo.current_germ_threat },
								 ipGeo.receivedGermData,
								 'json');
					  }, function() {});
};

ipGeo.inhale = function() {
	alert("Breathing in nice and deep. It's like you're licking a subway pole.");
};

ipGeo.cough = function() {

	ipGeo.getLocation(function(position) {
						  var lat = position.coords.latitude;
						  var lon = position.coords.longitude;
						  // recenter the map
						  ipGeo.map.setCenter(new GLatLng(lat, lon), 17);
						  $.post('/cough/',
								 { 'lat': lat,
								   'lon': lon,
								   'target_germ_id': ipGeo.target_germ_id },
								   ipGeo.receivedGermData,
								  'json');

					  }, function() {});

};

ipGeo.noLocation = function(response) {
	// log this?
};

ipGeo.commandCentralPoll = function() {
	$.post('/get/nearby/germs/',
	{ 'lat': ipGeo.default_lat, 'lon': ipGeo.default_lon },
		function(data) {
			ipGeo.receivedGermData(data);
		},
	'json');
	setTimeout(ipGeo.commandCentralPoll, 5000);
};

ipGeo.receivedGermData = function(data) {
	var message = data.message;
	var germs = data.germs;

	if(message) {
		alert(message);
	}

	ipGeo.current_germ_threat = '';
	for(var i in germs) {
		if(germs[i].threat)
			ipGeo.current_germ_threat = germs[i].id;

		ipGeo.putGermOnMap(germs[i]);
	}
};

ipGeo.showNearbyGerms = function(lat, lon) {
	$.post('/get/nearby/germs/',
		   { 'lat': lat, 'lon': lon },
		   ipGeo.receivedGermData,
		   'json');
};

ipGeo.challenge = function() {
	// hide the fullscreen viewer if it's open
	ipGeo.fullscreenHide();

	// show the message window and focus the text input
	$('#messageWindow').show();
	$('#msg').focus();

	// listen for a submit event
	$('#msg-form').submit(function() {

							 $.post('/message/to/germ/host',
									{ 'target_germ_id': ipGeo.current_germ_threat,
									  'msg': $('#msg').val()
									},

									// on successful submit, clear and hide the form
									function() {
										$('#msg').val('');
										$('#msg-send').unbind('click');
										$('#messageWindow').hide();
										$('#msg').blur();
									}
								   );
							 return false;
						 });
};

ipGeo.putGermOnMap = function(germ) {
	var lat = germ.lat;
	var lon = germ.lon;
	var spread = germ.spread;
	var color = germ.color;

	ipGeo.drawCircle(new GLatLng(lat, lon), spread, color);

	if(germ.threat) {
		var options = "<li><strong>You are Threatened!</strong><br />";
		options += "<a href='#' onclick=\"ipGeo.challenge();\">[CHALLENGE]</a> | ";
		options += "<a href='#' onclick=\"ipGeo.cover();\">[COVER UP]</a><br /><br />";
		$('#options').html(options);
	}

	var baseIcon = new GIcon(G_DEFAULT_ICON);
	baseIcon.iconSize = new GSize(107 / 3, 250 / 3 );
	baseIcon.shadow = false;
	baseIcon.image = "/static/images/purple-250.png";

	// Set up our GMarkerOptions object
	markerOptions = { icon:baseIcon };
	var marker = new GMarker(new GLatLng(lat, lon), markerOptions);
	GEvent.addListener(marker, 'click', function() {
						   var d = germ.disease;
						   html = "<div id='"+ germ.id + "'>";
						   if(germ.threat) {
							   html += "<strong>This germ is a threat to you!</strong><br>";
							   html += "Options: <br><ul>";

							   html += "<li><a href='#' onclick=\"ipGeo.challenge();\">[CHALLENGE]</a> (Send the user a message, if they don't respond it time, you're golden!)";
							   html += "<li><a href='#' onclick=\"ipGeo.cover();\">[COVER UP]</a> (play it safe)";

						   }

						   if(d.name) html += "<br><strong>Name</strong>: " + d.name;
						   html += "<br><small>";
						   html += "<strong>Strain</strong>: " + d.strain + "<br>";
						   html += "<strong>Host</strong>: " + d.host + "<br>";
//						   if(!d.host_is_you)
//							   html += "<a href='javascript://' onclick='ipGeo.initMessage(\"" + germ.id + "\");'>Send message to Host</a> (This is how you become host of this disease.).<br>";
						   html += "</div>";

						   ipGeo.fullscreenShow(html);
					   });

	ipGeo.map.addOverlay(marker);

};

// Draw a circle on map around center (radius in miles)
// Modified by Jeremy Schneider based on http://maps.huge.info/dragcircle2.htm
ipGeo.drawCircle = function(center, radius, color) {
    var poly = [];
    var lat = center.lat() ;
    var lng = center.lng() ;
    var d2r = Math.PI/180 ;                // degrees to radians
    var r2d = 180/Math.PI ;                // radians to degrees
    var Clat = (radius/3963) * r2d ;      //  using 3963 as earth's radius
    var Clng = Clat/Math.cos(lat*d2r);

    //Add each point in the circle
    for (var i = 0 ; i < ipGeo.points_in_circle ; i++)
    {
        var theta = Math.PI * (i / (ipGeo.points_in_circle / 2)) ;
        Cx = lng + (Clng * Math.cos(theta)) ;
        Cy = lat + (Clat * Math.sin(theta)) ;
        poly.push(new GLatLng(Cy,Cx)) ;
    }

    // //Add the first point to complete the circle
	poly.push(poly[0]) ;

    //Create a line with teh points from poly, red, 3 pixels wide, 80% opaque
//	var line = new GPolyline(poly,'#FF0000', 3, 0.9) ;
	var line = new GPolygon(poly, color, 3, 1, color, 0.2);
    ipGeo.map.addOverlay(line) ;
};


