// ask google to load the jquery libraries
google.load("jquery", "1");

var cc = {
	map: false,
	default_lat: 40.730521,
	default_lon: -73.984337,
	points_in_circle: 90,
	chat_cursor: ''
};

cc.initialize = function(map_div_id) {
	$('#chat-input').bind('keypress', function(e) {
							  var code = (e.keyCode ? e.keyCode : e.which);
							  if (code == 13) {
								  console.log("sending");
								  cc.sendChat();
							  }
						  });
    if (GBrowserIsCompatible()) {
        cc.map = new GMap2(document.getElementById(map_div_id));
        cc.map.setCenter(new GLatLng(cc.default_lat, cc.default_lon), 13);
		cc.map.setUIToDefault();
		cc.commandCentralPoll();
	}
	else
		alert("Google doesn't think your browser is compatible with their maps, this will be painful without them...");
};

cc.sendChat = function() {
	var text = $('#chat-input').val();
	$.post('/chat/say',
		{ 'text': text },
		function(data) {
			$('#chat-input').val('');
		},
		'json');
};

cc.commandCentralPoll = function() {
	$.post('/updates/',
		{ 'chat_cursor': cc.chat_cursor },
		function(data) {
			cc.receivedData(data);
			cc.commandCentralPoll();
		},
	'json');
};

cc.receivedData = function(data) {
	if(!data) return;

	var chat_messages = data.chat_messages;
	var germs = data.germs;

	// put the germs on the map
	if(germs) {
		// clear the current map
		cc.map.clearOverlays();
		for(var i in germs) {
			if(!germs[i].deleted)
				cc.putGermOnMap(germs[i]);
		}
	}

	if(chat_messages) {
		for(var x in chat_messages) {
			cc.chat_cursor = chat_messages[x].id;
			cc.addToChat(chat_messages[x]);
		}
	}

};

cc.addToChat = function(message) {
	var span = $('<div>');
	span.attr('class', 'single-message');
	span.html(message.text);
	$('#chat').prepend(span);
};

cc.putGermOnMap = function(germ) {
	var lat = germ.lat;
	var lon = germ.lon;
	var spread = germ.spread;
	var color = germ.color;

	cc.drawCircle(new GLatLng(lat, lon), spread, color);

	var baseIcon = new GIcon(G_DEFAULT_ICON);
	baseIcon.iconSize = new GSize(107 / 3, 250 / 3 );
	baseIcon.shadow = false;
	baseIcon.image = "http://www.ipandemic.com/static/images/purple-250.png";

	// Set up our GMarkerOptions object
	markerOptions = { icon:baseIcon };
	var marker = new GMarker(new GLatLng(lat, lon), markerOptions);
	GEvent.addListener(marker, 'click', function() {
						   var d = germ.disease;
						   html = "<div id='"+ germ.id + "'>";

						   if(d.name) html += "<br><strong>Name</strong>: " + d.name + "<br>";
						   html += "<small>";
						   html += "<strong>Strain</strong>: " + d.strain + "<br>";
						   html += "<strong>Host</strong>: " + d.host + "<br>";
						   html += "</div>";

						   marker.openInfoWindowHtml(html);

					   });

	cc.map.addOverlay(marker);

};

// Draw a circle on map around center (radius in miles)
// Modified by Jeremy Schneider based on http://maps.huge.info/dragcircle2.htm
cc.drawCircle = function(center, radius, color) {
    var poly = [];
    var lat = center.lat() ;
    var lng = center.lng() ;
    var d2r = Math.PI/180 ;                // degrees to radians
    var r2d = 180/Math.PI ;                // radians to degrees
    var Clat = (radius/3963) * r2d ;      //  using 3963 as earth's radius
    var Clng = Clat/Math.cos(lat*d2r);

    //Add each point in the circle
    for (var i = 0 ; i < cc.points_in_circle ; i++)
    {
        var theta = Math.PI * (i / (cc.points_in_circle / 2)) ;
        Cx = lng + (Clng * Math.cos(theta)) ;
        Cy = lat + (Clat * Math.sin(theta)) ;
        poly.push(new GLatLng(Cy,Cx)) ;
    }

    // //Add the first point to complete the circle
	poly.push(poly[0]) ;

    //Create a line with teh points from poly, red, 3 pixels wide, 80% opaque
//	var line = new GPolyline(poly,'#FF0000', 3, 0.9) ;
	var line = new GPolygon(poly, color, 3, 1, color, 0.2);
    cc.map.addOverlay(line) ;
};
